#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii
try:
    import tango
except Exception:
    import PyTango as tango
from nxstools import nxscreate


try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO


if sys.version_info > (3,):
    unicode = str
    long = int


# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreateCompareTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            import time
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        self.__args = '{"db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        # home = expanduser("~")
        db = tango.Database()
        self.host = db.get_db_host().split(".")[0]
        self.port = db.get_db_port()
        self.directory = "."
        self.flags = ""

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        print("SEED = %s" % self.seed)

    # test closer
    # \brief Common tear down
    def tearDown(self):
        print("tearing down ...")

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        nxscreate.main()
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
        except exception:
            error = True
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er

    def myAssertDict(self, dct, dct2):
        self.assertTrue(isinstance(dct, dict))
        self.assertTrue(isinstance(dct2, dict))
        self.assertEqual(
            len(list(dct.keys())), len(list(dct2.keys())))
        for k, v in dct.items():
            self.assertTrue(k in dct2.keys())
            if isinstance(v, dict):
                self.myAssertDict(v, dct2[k])
            else:
                self.assertEqual(v, dct2[k])

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    def test_compare_self(self):
        """ test nxsccreate compare file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname = '%s/%s%s.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
</hw>
"""
        commands = [('nxscreate compare %s %s %s'
                     % (fname, fname, self.flags)).split()]

        if os.path.isfile(fname):
            raise Exception("Test file %s exists" % fname)
        with open(fname, "w") as fl:
            fl.write(xml)
        try:

            for cmd in commands:
                vl, er = self.runtest(cmd)
                lines = vl.split("\n")
                self.assertEqual(len(lines), 14)
                self.assertTrue(lines[0].startswith("Comparing:"))
                self.assertEqual(lines[1], "")
                self.assertEqual(lines[3], "")
                self.assertEqual(lines[5], "")
                self.assertEqual(lines[7], "")
                self.assertEqual(lines[9], "")
                self.assertEqual(lines[11], "")
                self.assertEqual(lines[13], "")
                self.assertTrue(
                    lines[2].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname))
                self.assertTrue(
                    lines[6].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname))
                self.assertTrue(
                    lines[10].startswith(
                        "Diffrences in the common part:"))

                self.assertEqual(lines[4], "{}")
                self.assertEqual(lines[8], "{}")
                self.assertEqual(lines[12], "{}")

                # print(vl)
                # print(er)
        finally:
            os.remove(fname)

    def test_compare_diff(self):
        """ test nxsccreate compare file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname1 = '%s/%s%s_1.xml' % (
            os.getcwd(), self.__class__.__name__, fun)
        fname2 = '%s/%s%s_2.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml1 = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.04</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_vfcadc</name>
    <type>type_tango</type>
    <module>vfcadc</module>
    <device>mytest/vfcadc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        xml2 = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot_01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_tip850dac</name>
    <type>type_tango</type>
    <module>tip850dac</module>
    <device>mytest/tip850dac/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip830</name>
    <type>type_tango</type>
    <module>tip830</module>
    <device>mytest/tip830/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip850adc</name>
    <type>type_tango</type>
    <module>tip850adc</module>
    <device>mytest/tip850adc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        commands = [('nxscreate compare %s %s %s'
                     % (fname1, fname2, self.flags)).split()]

        if os.path.isfile(fname1):
            raise Exception("Test file %s exists" % fname1)
        elif os.path.isfile(fname2):
            raise Exception("Test file %s exists" % fname2)
        with open(fname1, "w") as fl:
            fl.write(xml1)
        with open(fname2, "w") as fl:
            fl.write(xml2)
        try:

            for cmd in commands:
                vl, er = self.runtest(cmd)
                lines = vl.split("\n\n")
                # print(vl)
                # print(er)
                self.assertEqual(len(lines), 7)
                self.assertTrue(lines[0].startswith("Comparing:"))
                self.assertTrue(
                    lines[1].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname1))
                self.assertTrue(
                    lines[3].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname2))
                self.assertTrue(
                    lines[5].startswith(
                        "Diffrences in the common part:"))

                first = eval(lines[2])
                second = eval(lines[4])
                common = eval(lines[6])
                self.myAssertDict(
                    first,
                    {'mot01': ['my_exp_mot01'],
                     'my_test_vfcadc': ['my_test_vfcadc']}
                )
                self.myAssertDict(
                    second,
                    {'mot_01': ['my_exp_mot01'],
                     'my_test_tip830': ['my_test_tip830'],
                     'my_test_tip850adc': ['my_test_tip850adc'],
                     'my_test_tip850dac': ['my_test_tip850dac']}
                )
                self.myAssertDict(
                    common,
                    {
                        'my_exp_mot03': [
                            {'dtype': ('motor', 'stepping_motor')}
                        ],
                        'my_exp_mot02': [
                            {
                                'tdevice':
                                ('p09/motor/exp.04', 'p09/motor/exp.02')
                            }
                        ],
                    }
                )

        finally:
            os.remove(fname1)
            os.remove(fname2)

    def test_compare_lower(self):
        """ test nxsccreate compare file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname1 = '%s/%s%s_1.xml' % (
            os.getcwd(), self.__class__.__name__, fun)
        fname2 = '%s/%s%s_2.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml1 = """<?xml version="1.0"?>
<hw>
<device>
 <name>My_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.04</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>Haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_vfcadc</name>
    <type>type_tango</type>
    <module>vfcadc</module>
    <device>mytest/vfcadc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        xml2 = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot_01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/Motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/Exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_tip850dac</name>
    <type>type_tango</type>
    <module>tip850dac</module>
    <device>mytest/tip850dac/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip830</name>
    <type>type_tango</type>
    <module>tip830</module>
    <device>Mytest/tip830/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>My_test_tip850adc</name>
    <type>type_tango</type>
    <module>tip850adc</module>
    <device>mytest/tip850adc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        commands = [('nxscreate compare %s %s %s'
                     % (fname1, fname2, self.flags)).split()]

        if os.path.isfile(fname1):
            raise Exception("Test file %s exists" % fname1)
        elif os.path.isfile(fname2):
            raise Exception("Test file %s exists" % fname2)
        with open(fname1, "w") as fl:
            fl.write(xml1)
        with open(fname2, "w") as fl:
            fl.write(xml2)
        try:

            for cmd in commands:
                vl, er = self.runtest(cmd)
                lines = vl.split("\n\n")
                # print(vl)
                # print(er)
                self.assertEqual(len(lines), 7)
                self.assertTrue(lines[0].startswith("Comparing:"))
                self.assertTrue(
                    lines[1].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname1))
                self.assertTrue(
                    lines[3].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname2))
                self.assertTrue(
                    lines[5].startswith(
                        "Diffrences in the common part:"))

                first = eval(lines[2])
                second = eval(lines[4])
                common = eval(lines[6])
                self.myAssertDict(
                    first,
                    {'my_test_vfcadc': ['my_test_vfcadc']})
                self.myAssertDict(
                    second,
                    {'my_test_tip830': ['my_test_tip830'],
                     'my_test_tip850adc': ['my_test_tip850adc'],
                     'my_test_tip850dac': ['my_test_tip850dac']})
                self.myAssertDict(
                    common,
                    {'mot01': [{'name': ('my_exp_mot01', 'my_exp_mot_01'),
                                'tdevice':
                                ('P09/motor/exp.01', 'p09/Motor/exp.01')}],
                     'my_exp_mot02':
                     [{'tdevice': ('P09/motor/exp.04', 'p09/motor/Exp.02')}],
                     'my_exp_mot03':
                     [{'dtype': ('motor', 'stepping_motor'),
                       'hostname': ('Haso000:10000', 'haso000:10000')}]}
                )

        finally:
            os.remove(fname1)
            os.remove(fname2)

    def test_compare_nolower(self):
        """ test nxsccreate compare file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        fname1 = '%s/%s%s_1.xml' % (
            os.getcwd(), self.__class__.__name__, fun)
        fname2 = '%s/%s%s_2.xml' % (
            os.getcwd(), self.__class__.__name__, fun)

        xml1 = """<?xml version="1.0"?>
<hw>
<device>
 <name>My_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>P09/motor/exp.04</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>Haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_vfcadc</name>
    <type>type_tango</type>
    <module>vfcadc</module>
    <device>mytest/vfcadc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        xml2 = """<?xml version="1.0"?>
<hw>
<device>
 <name>my_exp_mot01</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/Motor/exp.01</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>1</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
 <sardananame>mot_01</sardananame>
</device>
<device>
 <name>my_exp_mot02</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/Exp.02</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>2</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
 <name>my_exp_mot03</name>
 <type>stepping_motor</type>
 <module>oms58</module>
 <device>p09/motor/exp.03</device>
 <control>tango</control>
 <hostname>haso000:10000</hostname>
 <controller>oms58_exp</controller>
 <channel>3</channel>
 <rootdevicename>p09/motor/exp</rootdevicename>
</device>
<device>
    <name>my_test_tip850dac</name>
    <type>type_tango</type>
    <module>tip850dac</module>
    <device>mytest/tip850dac/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>my_test_tip830</name>
    <type>type_tango</type>
    <module>tip830</module>
    <device>Mytest/tip830/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
<device>
    <name>My_test_tip850adc</name>
    <type>type_tango</type>
    <module>tip850adc</module>
    <device>mytest/tip850adc/ct</device>
    <control>tango</control>
    <hostname>haso000:10000</hostname>
</device>
</hw>
"""
        commands = [
            ('nxscreate compare -n %s %s %s'
             % (fname1, fname2, self.flags)).split(),
            ('nxscreate compare --nolower %s %s %s'
             % (fname1, fname2, self.flags)).split()
        ]

        if os.path.isfile(fname1):
            raise Exception("Test file %s exists" % fname1)
        elif os.path.isfile(fname2):
            raise Exception("Test file %s exists" % fname2)
        with open(fname1, "w") as fl:
            fl.write(xml1)
        with open(fname2, "w") as fl:
            fl.write(xml2)
        try:

            for cmd in commands:
                vl, er = self.runtest(cmd)
                lines = vl.split("\n\n")
                # print(vl)
                # print(er)
                self.assertEqual(len(lines), 7)
                self.assertTrue(lines[0].startswith("Comparing:"))
                self.assertTrue(
                    lines[1].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname1))
                self.assertTrue(
                    lines[3].startswith(
                        "Additional devices in '%s' {alias: [name]} :"
                        % fname2))
                self.assertTrue(
                    lines[5].startswith(
                        "Diffrences in the common part:"))

                first = eval(lines[2])
                second = eval(lines[4])
                common = eval(lines[6])
                self.myAssertDict(
                    first,
                    {'mot01': ['My_exp_mot01'],
                     'my_test_vfcadc': ['my_test_vfcadc']}
                )
                self.myAssertDict(
                    second,
                    {'My_test_tip850adc': ['My_test_tip850adc'],
                     'mot_01': ['my_exp_mot01'],
                     'my_test_tip830': ['my_test_tip830'],
                     'my_test_tip850dac': ['my_test_tip850dac']}
                    )
                self.myAssertDict(
                    common,
                    {'my_exp_mot02':
                     [{'tdevice': ('P09/motor/exp.04', 'p09/motor/Exp.02')}],
                     'my_exp_mot03':
                     [{'dtype': ('motor', 'stepping_motor'),
                       'hostname': ('Haso000:10000', 'haso000:10000')}]}
                )

        finally:
            os.remove(fname1)
            os.remove(fname2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2014 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file ServerSetUp.py
# class with server settings
#
import os
import sys
import subprocess
import shutil

try:
    import tango
except Exception:
    import PyTango as tango
import time
try:
    import TestPool
except Exception:
    from . import TestPool


# test fixture
class TestServerSetUp(object):

    # constructor
    # \brief defines server parameters

    def __init__(self, device="ttestp09/testts/t1r228", instance="S1",
                 alias=None):
        # information about tango writer
        self.new_device_info_writer = tango.DbDevInfo()
        # information about tango writer class
        self.new_device_info_writer._class = "TestServer"
        # information about tango writer server
        self.new_device_info_writer.server = "TestServer/%s" % instance
        # information about tango writer name
        self.new_device_info_writer.name = device

        # server instance
        self.instance = instance
        self._psub = None
        self._alias = alias
        # device proxy
        self.dp = None
        # device properties
        self.device_prop = {
            'DeviceBoolean': False,
            'DeviceShort': 12,
            'DeviceLong': 1234566,
            'DeviceFloat': 12.4345,
            'DeviceDouble': 3.453456,
            'DeviceUShort': 1,
            'DeviceULong': 23234,
            'DeviceString': "My Sting"
        }

        # class properties
        self.class_prop = {
            'ClassBoolean': True,
            'ClassShort': 1,
            'ClassLong': -123555,
            'ClassFloat': 12.345,
            'ClassDouble': 1.23445,
            'ClassUShort': 1,
            'ClassULong': 12343,
            'ClassString': "My ClassString",
        }

    # test starter
    # \brief Common set up of Tango Server
    def setUp(self):
        print("\nsetting up...")
        path = os.path.dirname(TestPool.__file__)
        if sys.version_info > (3,):
            shutil.copy2("%s/TestServer3" % path, "%s/TestServer" % path)
        else:
            shutil.copy2("%s/TestServer2" % path, "%s/TestServer" % path)
        self.add()
        self.start()

    def add(self):
        db = tango.Database()
        db.add_device(self.new_device_info_writer)
        db.add_server(self.new_device_info_writer.server,
                      self.new_device_info_writer)
        db.put_device_property(self.new_device_info_writer.name,
                               self.device_prop)
        db.put_class_property(self.new_device_info_writer._class,
                              self.class_prop)
        if self._alias:
            db.put_device_alias(
                self.new_device_info_writer.name, self._alias)

    # starts server
    def start(self):
        db = tango.Database()
        path = os.path.dirname(TestPool.__file__)
        if not path:
            path = '.'

        if sys.version_info > (3,):
            self._psub = subprocess.call(
                "cd %s; python3 ./TestServer %s &" %
                (path, self.instance), stdout=None,
                stderr=None, shell=True)
        else:
            self._psub = subprocess.call(
                "cd %s; python ./TestServer %s &" %
                (path, self.instance), stdout=None,
                stderr=None, shell=True)

        sys.stdout.write("waiting for simple server")

        found = False
        cnt = 0
        dvname = self.new_device_info_writer.name
        while not found and cnt < 1000:
            try:
                sys.stdout.write(".")
                sys.stdout.flush()
                exl = db.get_device_exported(dvname)
                if dvname not in exl.value_string:
                    time.sleep(0.01)
                    cnt += 1
                    continue
                self.dp = tango.DeviceProxy(dvname)
                time.sleep(0.01)
                if self.dp.state() == tango.DevState.ON:
                    found = True
            except Exception:
                found = False
            cnt += 1
        print("")

    # test closer
    # \brief Common tear down of Tango Server
    def tearDown(self):
        print("tearing down ...")
        self.delete()
        self.stop()
        path = "%s/TestServer" % os.path.dirname(TestPool.__file__)
        if os.path.exists(path):
            os.remove(path)

    def delete(self):
        db = tango.Database()
        if self._alias:
            db.delete_device_alias(self._alias)
        db.delete_server(self.new_device_info_writer.server)

    # stops server
    def stop(self):
        if sys.version_info > (3,):
            with subprocess.Popen(
                    "ps -ef | grep 'TestServer %s' | grep -v grep" %
                    self.instance,
                    stdout=subprocess.PIPE, shell=True) as proc:

                pipe = proc.stdout
                res = str(pipe.read(), "utf8").split("\n")
                for r in res:
                    sr = r.split()
                    if len(sr) > 2:
                        subprocess.call(
                            "kill -9 %s" % sr[1], stderr=subprocess.PIPE,
                            shell=True)
                pipe.close()
        else:
            pipe = subprocess.Popen(
                "ps -ef | grep 'TestServer %s' | grep -v grep" %
                self.instance,
                stdout=subprocess.PIPE, shell=True).stdout

            res = str(pipe.read()).split("\n")
            for r in res:
                sr = r.split()
                if len(sr) > 2:
                    subprocess.call(
                        "kill -9 %s" % sr[1], stderr=subprocess.PIPE,
                        shell=True)
            pipe.close()


# test fixture
class MultiTestServerSetUp(object):

    # constructor
    # \brief defines server parameters

    def __init__(self, instance="MTS01", devices=None):
        if not isinstance(devices, list):
            devices = ["ttestp09/testts/mt1r228"]

        # information about tango writer
        self.server = "TestServer/%s" % instance
        self.ts = {}
        # device proxy
        self.dps = {}
        for dv in devices:
            self.ts[dv] = tango.DbDevInfo()
            self.ts[dv]._class = "TestServer"
            self.ts[dv].server = self.server
            self.ts[dv].name = dv

        # server instance
        self.instance = instance
        self._psub = None

        # device properties
        self.device_prop = {
            'DeviceBoolean': False,
            'DeviceShort': 12,
            'DeviceLong': 1234566,
            'DeviceFloat': 12.4345,
            'DeviceDouble': 3.453456,
            'DeviceUShort': 1,
            'DeviceULong': 23234,
            'DeviceString': "My Sting"
        }

        # class properties
        self.class_prop = {
            'ClassBoolean': True,
            'ClassShort': 1,
            'ClassLong': -123555,
            'ClassFloat': 12.345,
            'ClassDouble': 1.23445,
            'ClassUShort': 1,
            'ClassULong': 12343,
            'ClassString': "My ClassString",
        }

    # test starter
    # \brief Common set up of Tango Server
    def setUp(self):
        print("\nsetting up...")
        self.add()
        self.start()

    def add(self):
        db = tango.Database()

        devices = list(self.ts.values())
        for dv in devices:
            db.add_device(dv)
            # print dv.name
            db.put_device_property(dv.name, self.device_prop)
            db.put_class_property(dv._class, self.class_prop)

        if devices:
            db.add_server(self.server, devices)

    # starts server
    def start(self):
        db = tango.Database()
        path = os.path.dirname(TestPool.__file__)
        if not path:
            path = '.'

        if sys.version_info > (3,):
            self._psub = subprocess.call(
                "cd %s;  python3 ./TestServer %s &" % (path, self.instance),
                stdout=None, stderr=None, shell=True)
        else:
            self._psub = subprocess.call(
                "cd %s;  python ./TestServer %s &" % (path, self.instance),
                stdout=None, stderr=None, shell=True)
        sys.stdout.write("waiting for simple server")

        found = False
        cnt = 0
        devices = list(self.ts.values())
        while not found and cnt < 1000:
            try:
                sys.stdout.write(".")
                dpcnt = 0
                for dv in devices:
                    exl = db.get_device_exported(dv.name)
                    if dv.name not in exl.value_string:
                        time.sleep(0.01)
                        cnt += 1
                        continue
                    self.dps[dv.name] = tango.DeviceProxy(dv.name)
                    time.sleep(0.01)
                    if self.dps[dv.name].state() == tango.DevState.ON:
                        dpcnt += 1
                if dpcnt == len(devices):
                    found = True
            except Exception:
                found = False
            cnt += 1
        print("")

    # test closer
    # \brief Common tear down of Tango Server
    def tearDown(self):
        print("tearing down ...")
        self.delete()
        self.stop()

    def delete(self):
        db = tango.Database()
        db.delete_server(self.server)

    # stops server
    def stop(self):
        if sys.version_info > (3,):
            with subprocess.Popen(
                    "ps -ef | grep 'TestServer %s' | grep -v grep" %
                    self.instance,
                    stdout=subprocess.PIPE, shell=True) as proc:

                pipe = proc.stdout
                res = str(pipe.read(), "utf8").split("\n")
                for r in res:
                    sr = r.split()
                    if len(sr) > 2:
                        subprocess.call(
                            "kill -9 %s" % sr[1], stderr=subprocess.PIPE,
                            shell=True)
                pipe.close()
        else:
            pipe = subprocess.Popen(
                "ps -ef | grep 'TestServer %s' | grep -v grep" %
                self.instance,
                stdout=subprocess.PIPE, shell=True).stdout

            res = str(pipe.read()).split("\n")
            for r in res:
                sr = r.split()
                if len(sr) > 2:
                    subprocess.call(
                        "kill -9 %s" % sr[1], stderr=subprocess.PIPE,
                        shell=True)
            pipe.close()


if __name__ == "__main__":
    simps = TestServerSetUp()
    simps.setUp()
    print(simps.dp.status())
    simps.tearDown()
    simps = MultiTestServerSetUp()
    simps.setUp()
    for dp in simps.dps.values():
        print(dp.status())
    simps.tearDown()
    simps = MultiTestServerSetUp(devices=[
        "tm2/dd/sr", "dsf/44/fgg", "sdffd/sdfsd/sfd"])
    simps.setUp()
    for dp in simps.dps.values():
        print(dp.status())
    simps.tearDown()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file runtest.py
# the unittest runner
#

import os
import sys

try:
    __import__("PyTango")
    # if module PyTango available
    PYTANGO_AVAILABLE = True
except ImportError as e:
    PYTANGO_AVAILABLE = False
    print("PyTango is not available: %s" % e)

try:
    __import__("h5py")
    # if module h5py available
    H5PY_AVAILABLE = True
except ImportError as e:
    H5PY_AVAILABLE = False
    print("h5py is not available: %s" % e)

try:
    __import__("pninexus.h5cpp")
    # if module pninexus.h5cpp available
    H5CPP_AVAILABLE = True
except ImportError as e:
    H5CPP_AVAILABLE = False
    print("h5cpp is not available: %s" % e)
except SystemError as e:
    H5CPP_AVAILABLE = False
    print("h5cpp is not available: %s" % e)


import unittest

import NXSTools_test

if not H5PY_AVAILABLE and not H5CPP_AVAILABLE:
    raise Exception("Please install h5py or pninexus.h5cpp")


# list of available databases
DB_AVAILABLE = []

try:
    import MySQLdb
    # connection arguments to MYSQL DB
    args = {}
    args["db"] = 'tango'
    # args["host"] = 'localhost'
    args["read_default_file"] = '/etc/mysql/my.cnf'
    # inscance of MySQLdb
    print(args)
    mydb = MySQLdb.connect(**args)
    mydb.close()
    DB_AVAILABLE.append("MYSQL")
except Exception as e1:
    try:
        import MySQLdb
        from os.path import expanduser
        home = expanduser("~")
        # connection arguments to MYSQL DB
        cnffile = '%s/.my.cnf' % home
        args2 = {
            # 'host': u'localhost',
            'db': u'tango',
            'read_default_file': '%s/.my.cnf' % home,
            'use_unicode': True}
        # inscance of MySQLdb
        print(args2)
        mydb = MySQLdb.connect(**args2)
        mydb.close()
        DB_AVAILABLE.append("MYSQL")
    except ImportError as e2:
        print("MYSQL not available: %s %s" % (e1, e2))
    except Exception as e2:
        print("MYSQL not available: %s %s" % (e1, e2))
    except Exception:
        print("MYSQL not available")


try:
    import psycopg2
    # connection arguments to PGSQL DB
    args = {}
    args["database"] = 'mydb'
    # inscance of psycog2
    pgdb = psycopg2.connect(**args)
    pgdb.close()
    DB_AVAILABLE.append("PGSQL")
except ImportError as e:
    print("PGSQL not available: %s" % e)
except Exception as e:
    print("PGSQL not available: %s" % e)
except Exception:
    print("PGSQL not available")


try:
    import cx_Oracle
    # pwd
    with open('%s/pwd' % os.path.dirname(NXSTools_test.__file__)) as fl:
        passwd = fl.read()[:-1]

    # connection arguments to ORACLE DB
    args = {}
    args["dsn"] = (
        "(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST=dbsrv01.desy.de)"
        "(PORT=1521))(LOAD_BALANCE=yes)(CONNECT_DATA=(SERVER=DEDICATED)"
        "(SERVICE_NAME=desy_db.desy.de)(FAILOVER_MODE=(TYPE=NONE)"
        "(METHOD=BASIC)(RETRIES=180)(DELAY=5))))")
    args["user"] = "read"
    args["password"] = passwd
    # inscance of cx_Oracle
    ordb = cx_Oracle.connect(**args)
    ordb.close()
    DB_AVAILABLE.append("ORACLE")
except ImportError as e:
    print("ORACLE not available: %s" % e)
except Exception as e:
    print("ORACLE not available: %s" % e)
except Exception:
    print("ORACLE not available")


if H5PY_AVAILABLE:
    import H5PYWriter_test
    import FileWriterH5PY_test
    import NXSCollectH5PY_test
    import NXSFileInfoH5PY_test
    import NXSCreatePyEvalH5PY_test
if H5CPP_AVAILABLE:
    import H5CppWriter_test
    import NXSCreatePyEvalH5Cpp_test
    import FileWriterH5Cpp_test
    import NXSCollectH5Cpp_test
    import NXSFileInfoH5Cpp_test
    import H5CppRedisWriter_test
if H5PY_AVAILABLE and H5CPP_AVAILABLE:
    import FileWriterH5CppH5PY_test

if H5CPP_AVAILABLE or H5PY_AVAILABLE:
    import NXSCollect_test
    import NXSFileInfo_test


if PYTANGO_AVAILABLE:
    import NXSCreateClientDSFS_test
    import NXSCreateClientDSFS2_test
    import NXSCreateClientDSFS3_test
    import NXSCreate_test
    import NXSCreateCompare_test

    import NXSCreateTangoDSFS_test
    import NXSCreateTangoDSFS2_test
    import NXSCreateTangoDSFS3_test

    import NXSCreateDeviceDSFS_test
    import NXSCreateDeviceDSFS2_test
    import NXSCreateDeviceDSFS3_test

    import NXSCreateCompFS_test
    import NXSCreateCompFS2_test
    import NXSCreateCompFS3_test

    import NXSCreateOnlineDSFS_test
    import NXSCreateOnlineDSFS2_test
    import NXSCreateOnlineDSFS3_test

    import NXSCreateOnlineCPFS_test
    import NXSCreateOnlineCPFS2_test
    import NXSCreateOnlineCPFS3_test

    import NXSCreatePoolDSFS_test
    import NXSCreatePoolDSFS2_test
    import NXSCreatePoolDSFS3_test

    import NXSCreateSECoPCPFS_test

    import NXSData_test

    if "MYSQL" in DB_AVAILABLE:
        import NXSetUp_test

        import NXSCreateStdCompFS_test
        import NXSCreateStdCompFS2_test
        import NXSCreateStdCompFS3_test

        import NXSConfig_test
        import NXSCreateClientDSDB_test
        import NXSCreateClientDSDB2_test
        import NXSCreateClientDSDBR_test
        import NXSCreateClientDSDBR2_test

        import NXSCreateTangoDSDB_test
        import NXSCreateTangoDSDB2_test
        import NXSCreateTangoDSDBR_test
        import NXSCreateTangoDSDBR2_test

        import NXSCreateCompDB_test
        import NXSCreateCompDB2_test
        import NXSCreateCompDBR_test
        import NXSCreateCompDBR2_test

        import NXSCreateDeviceDSDB_test
        import NXSCreateDeviceDSDB2_test
        import NXSCreateDeviceDSDBR_test
        import NXSCreateDeviceDSDBR2_test
        import NXSCreateDeviceDSFS4_test

        import NXSCreateOnlineDSDB_test
        import NXSCreateOnlineDSDB2_test
        import NXSCreateOnlineDSDBR_test
        import NXSCreateOnlineDSDBR2_test
        import NXSCreateOnlineDSDBE_test
        import NXSCreateOnlineDSDBE2_test

        import NXSCreateOnlineCPDB_test
        import NXSCreateOnlineCPDB2_test
        import NXSCreateOnlineCPDBR_test
        import NXSCreateOnlineCPDBR2_test

        import NXSCreateStdCompDB_test
        import NXSCreateStdCompDB2_test
        import NXSCreateStdCompDBR_test
        import NXSCreateStdCompDBR2_test
        import NXSCreateStdCompDBE_test
        import NXSCreateStdCompDBE2_test

        import NXSCreatePoolDSDB_test
        import NXSCreatePoolDSDB2_test
        import NXSCreatePoolDSDBR_test
        import NXSCreatePoolDSDBR2_test

        import NXSCreateSECoPCPDB_test


# main function
def main():

    # test suit
    suite = unittest.TestSuite()

    if H5PY_AVAILABLE:
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                FileWriterH5PY_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                H5PYWriter_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCollectH5PY_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSFileInfoH5PY_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreatePyEvalH5PY_test))
    if H5CPP_AVAILABLE:
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                FileWriterH5Cpp_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                H5CppWriter_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCollectH5Cpp_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSFileInfoH5Cpp_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreatePyEvalH5Cpp_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                H5CppRedisWriter_test))
    if H5CPP_AVAILABLE and H5PY_AVAILABLE:
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                FileWriterH5CppH5PY_test))

    if H5CPP_AVAILABLE or H5PY_AVAILABLE or H5CPP_AVAILABLE:
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCollect_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSFileInfo_test))

    if PYTANGO_AVAILABLE:
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateClientDSFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateClientDSFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateClientDSFS3_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreate_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateCompare_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSData_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateTangoDSFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateTangoDSFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateTangoDSFS3_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineDSFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineDSFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineDSFS3_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineCPFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineCPFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateOnlineCPFS3_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreatePoolDSFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreatePoolDSFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreatePoolDSFS3_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateSECoPCPFS_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateDeviceDSFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateDeviceDSFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateDeviceDSFS3_test))

        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateCompFS_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateCompFS2_test))
        suite.addTests(
            unittest.defaultTestLoader.loadTestsFromModule(
                NXSCreateCompFS3_test))
        if "MYSQL" in DB_AVAILABLE:
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSetUp_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSConfig_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompFS_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompFS2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompFS3_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateClientDSDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateClientDSDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateClientDSDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateClientDSDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateTangoDSDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateTangoDSDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateTangoDSDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateTangoDSDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineCPDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineCPDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineCPDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineCPDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateSECoPCPDB_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDBE_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateOnlineDSDBE2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDBE_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateStdCompDBE2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreatePoolDSDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreatePoolDSDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreatePoolDSDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreatePoolDSDBR2_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateDeviceDSDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateDeviceDSDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateDeviceDSDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateDeviceDSDBR2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateDeviceDSFS4_test))

            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateCompDB_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateCompDB2_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateCompDBR_test))
            suite.addTests(
                unittest.defaultTestLoader.loadTestsFromModule(
                    NXSCreateCompDBR2_test))

    # test runner
    runner = unittest.TextTestRunner()
    # test result
    result = runner.run(suite).wasSuccessful()
    sys.exit(not result)

    #   if ts:
    #       ts.tearDown()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2017 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

"""  xml templates """


#: (:obj:`dict` <:obj:`str` , :obj:`dict` <:obj:`str` , :obj:`str` > >)
#:     standard component template variables
#:     and its [default value, doc string]
standardComponentVariables = {
    'collect4': {
        'first': {
            'default': None,
            'doc': "name of the first component to collect MANDATORY"
            " (datasource)"
        },
        'second': {
            'default': None,
            'doc': "name of the second component to collect MANDATORY"
            " (datasource)"
        },
        'third': {
            'default': None,
            'doc': "name of the third component to collect MANDATORY"
            " (datasource)"
        },
        'fourth': {
            'default': None,
            'doc': "name of the fourth component to collect MANDATORY"
            " (datasource)"
        }
    },
    'common4': {
        'dds': {
            'default': None,
            'doc': "default read datasource name MANDATORY (datasource)"
        },
        'ods1': {
            'default': None,
            'doc': "fist optional detasource name MANDATORY (datasource)"
        },
        'ods2': {
            'default': None,
            'doc': "second optional detasource name MANDATORY (datasource)"
        },
        'ods3': {
            'default': None,
            'doc': "third optional detasource name MANDATORY (datasource)"
        }
    },
}

#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     xml template files of modules
standardComponentTemplateFiles = {
    'collect4': [
        'collect4.xml',
    ],
    'common4': [
        'common4_common.ds.xml',
    ],
}

#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     xml template files of modules
moduleTemplateFiles = {
}

#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     important attributes of modules
moduleMultiAttributes = {
    'mca_xia_test': [
        'ICR', 'OCR',
    ],
    'mca_xia@pool_test': [
        'CountsRoI', 'RoIEnd', 'RoIStart',
    ],
    'limaccd_test': [
        'camera_type', 'camera_pixelsize', 'camera_model',
        'acq_mode', 'acq_nb_frames', 'acq_trigger_mode',
        'last_image_saved',
        'latency_time',  'acc_max_expo_time',
        'acc_expo_time', 'acc_time_mode',
        'acc_dead_time', 'acc_live_time',
        'image_width',
        'image_height',
        'image_sizes',
        'image_roi',
        'image_bin',
        'image_flip',
        'image_rotation',
        'shutter_mode',
        'shutter_open_time',
    ],
    'limaccds_test': [
        'camera_type', 'camera_pixelsize', 'camera_model',
        'acq_mode', 'acq_nb_frames', 'acq_trigger_mode',
        'last_image_saved',
        'latency_time', 'acc_max_expo_time',
        'acc_expo_time',  'acc_time_mode',
        'image_type',
        'image_width',
        'image_height',
        'image_sizes',
        'image_roi',
        'image_bin',
        'image_flip',
        'image_rotation',
        'shutter_mode',
        'shutter_open_time',
    ],
    'pco_test': [
        'DelayTime', 'ExposureTime', 'NbFrames', 'TriggerMode',
        'FileDir', 'FilePostfix', 'FilePrefix', 'FileStartNum',
        'CoolingTemp', 'CoolingTempSet', 'ImageTimeStamp',
        'RecorderMode',
    ],
}

#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     xml template files of modules
moduleTemplateFiles = {
    'mymca': [
        'mymca.xml'
    ],
}


#: (:obj:`dict` <:obj:`str` , :obj:`list` <:obj:`str`> >)
#:     important attributes of modules
moduleMultiAttributes = {
    'mymca': [
        'Data', 'Mode',
    ],
}



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import os
import sys
import random
import struct
import binascii
# import time
# import threading
try:
    import tango
except Exception:
    import PyTango as tango
# import json
from nxstools import nxscreate
from nxstools.xmltemplates import standardComponentVariables

try:
    from .checks import checkxmls
except Exception:
    from checks import checkxmls

try:
    import nxsextrasp00
except ImportError:
    from . import nxsextrasp00


try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

try:
    import ServerSetUp
except ImportError:
    from . import ServerSetUp


if sys.version_info > (3,):
    unicode = str
    long = int

if "nxsextrasp00" not in sys.modules:
    sys.modules["nxsextrasp00"] = nxsextrasp00

# if 64-bit machione
IS64BIT = (struct.calcsize("P") == 8)


# test fixture
class NXSCreateStdCompFSTest(unittest.TestCase):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        unittest.TestCase.__init__(self, methodName)

        try:
            # random seed
            self.seed = long(binascii.hexlify(os.urandom(16)), 16)
        except NotImplementedError:
            import time
            # random seed
            self.seed = long(time.time() * 256)  # use fractional seconds

        self._rnd = random.Random(self.seed)

        self._bint = "int64" if IS64BIT else "int32"
        self._buint = "uint64" if IS64BIT else "uint32"
        self._bfloat = "float64" if IS64BIT else "float32"

        self.__args = '{"db":"nxsconfig", ' \
                      '"read_default_file":"/etc/my.cnf", "use_unicode":true}'

        # home = expanduser("~")
        db = tango.Database()
        self.host = db.get_db_host().split(".")[0]
        self.port = db.get_db_port()
        self._sv = ServerSetUp.ServerSetUp()
        self.maxDiff = None
        self.directory = "."
        self.flags = " -r testp09/testmcs/testr228 "
        self.device = 'testp09/testmcs/testr228'

    # sets xmlconfiguration
    # \param xmlc configuration instance
    # \param xml xml configuration string
    def setXML(self, xmlc, xml):
        xmlc.XMLString = xml

    # gets xmlconfiguration
    # \param xmlc configuration instance
    # \returns xml configuration string
    def getXML(self, xmlc):
        return xmlc.XMLString

    # test starter
    # \brief Common set up
    def setUp(self):
        print("\nsetting up...")
        print("SEED = %s" % self.seed)
        self._sv.setUp()

    # test closer
    # \brief Common tear down
    def tearDown(self):
        print("tearing down ...")
        self._sv.tearDown()

    def dsexists(self, name):
        return os.path.isfile("%s/%s.ds.xml" % (self.directory, name))

    def cpexists(self, name):
        return os.path.isfile("%s/%s.xml" % (self.directory, name))

    def checkmandatory(self, name, mandat):
        pass

    def getds(self, name):
        with open("%s/%s.ds.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def getcp(self, name):
        with open("%s/%s.xml" % (self.directory, name), 'r') as fl:
            xml = fl.read()
        return xml

    def deleteds(self, name):
        os.remove("%s/%s.ds.xml" % (self.directory, name))

    def deletecp(self, name):
        os.remove("%s/%s.xml" % (self.directory, name))

    def runtest(self, argv):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        # try:
        #    err = False
        nxscreate.main()
        # except Exception as e:
        #     print(str(e))
        #     err = True
        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        # print(vl)
        # print(er)
        # if err:
        #     raise
        return vl, er

    def runtestexcept(self, argv, exception):
        old_stdout = sys.stdout
        old_stderr = sys.stderr
        sys.stdout = mystdout = StringIO()
        sys.stderr = mystderr = StringIO()

        old_argv = sys.argv
        sys.argv = argv
        try:
            error = False
            nxscreate.main()
            etxt = None
        except exception as e:
            error = True
            etxt = str(e)
        self.assertEqual(error, True)

        sys.argv = old_argv

        sys.stdout = old_stdout
        sys.stderr = old_stderr
        vl = mystdout.getvalue()
        er = mystderr.getvalue()
        return vl, er, etxt

    # Exception tester
    # \param exception expected exception
    # \param method called method
    # \param args list with method arguments
    # \param kwargs dictionary with method arguments
    def myAssertRaise(self, exception, method, *args, **kwargs):
        try:
            error = False
            method(*args, **kwargs)
        except exception:
            error = True
        self.assertEqual(error, True)

    def checkxmls(self, args, mandatory=False):
        """ check xmls of components and datasources
        """

        dstotest = []
        cptotest = []
        try:
            for arg in args:
                skip = False
                for cp in arg[1][0]:
                    if self.cpexists(cp):
                        skip = True
                for ds in arg[1][1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1][1]:
                        dstotest.append(ds)
                    for cp in arg[1][0]:
                        cptotest.append(cp)
                    # print(arg[0])
                    vl, er = self.runtest(arg[0])
                    # print(vl)
                    if er:
                        self.assertEqual(
                            "Info: NeXus hasn't been setup yet. \n\n", er)
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1][1]):
                        xml = self.getds(ds)
                        checkxmls(
                            self,
                            arg[2][1][i], xml)
                    for i, cp in enumerate(arg[1][0]):
                        xml = self.getcp(cp)
                        checkxmls(
                            self,
                            arg[2][0][i], xml)
                        self.checkmandatory(cp, mandatory)

                    for ds in arg[1][1]:
                        self.deleteds(ds)
                    for cp in arg[1][0]:
                        self.deletecp(cp)

        finally:
            pass
            for cp in cptotest:
                if self.cpexists(cp):
                    self.deletecp(cp)
            for ds in dstotest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_stdcomp_typelist(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp %s' % self.flags).split(),
            ],
        ]

        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                if er:
                    self.assertEqual(
                        "Info: NeXus hasn't been setup yet. \n\n", er)
                else:
                    self.assertEqual('', er)
                self.assertTrue(vl)
                lines = vl.split("\n")
                self.assertEqual(lines[-3], "POSSIBLE COMPONENT TYPES: ")
                self.assertEqual(
                    lines[-2].split(),
                    ["absorber", "beamstop", "beamtimefname",
                     "beamtimeid", "chcut", "coboldhisto",
                     "collect2", "collect3",
                     "collect4", "collect5", "collect6",
                     "common2", "common3",
                     "dataaxessignal",
                     "datasignal", "dcm", "default", "defaultinstrument",
                     "defaultsample", "description",
                     "descriptiontext", "detectorlive",
                     "empty", "groupsecop", "keithley", "maia",
                     "maiadimension", "maiaflux", "msnsar", "mssar",
                     "parametercopymap", "pinhole",
                     "pointdet", "qbpm", "sampledescription",
                     "sampledescriptiontext", "samplehkl", "secop",
                     "secoplinks",
                     "singlesecop", "slit", "source", "starttime",
                     "tango", "undulator"])
                self.assertEqual(lines[-1], "")
        finally:
            pass

    def test_stdcomp_type_parameters(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        types = ["absorber", "beamstop", "beamtimeid", "chcut",
                 "coboldhisto", "collect2", "collect3",
                 "collect4", "collect5", "collect6",
                 "common2", "common3",
                 "dataaxessignal",
                 "datasignal", "dcm", "default", "defaultinstrument",
                 "defaultsample", "description",
                 "descriptiontext", "detectorlive",
                 "empty", "groupsecop", "keithley", "maia",
                 "maiadimension", "maiaflux",
                 "parametercopymap", "pinhole",
                 "pointdet", "qbpm", "sampledescription",
                 "sampledescriptiontext", "samplehkl",
                 "secop", "secoplinks", "singlesecop", "slit",
                 "source", "starttime", "tango", "undulator"]
        args = [
                ('nxscreate stdcomp %s -t ' % self.flags).split(),
                ('nxscreate stdcomp %s --type ' % self.flags).split(),
        ]

        try:
            for tp in types:
                for arg in args:
                    cmd = list(arg)
                    cmd.append(tp)
                    vl, er = self.runtest(cmd)

                    if er:
                        self.assertEqual(
                            "Info: NeXus hasn't been setup yet. \n\n", er)
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    lines = vl.split("\n")
                    self.assertEqual(lines[0], "")
                    self.assertEqual(lines[-1], "")
                    self.assertEqual(lines[1], "COMPONENT VARIABLES:")
                    var = lines[2:-1]

                    self.assertEqual(
                        len(var),
                        len([st for st in standardComponentVariables[tp].keys()
                             if not st.startswith("__")]))
                    for vr in var:
                        vname = vr.split()[0]
                        self.assertTrue(
                             vname in standardComponentVariables[tp].keys())
                        self.assertTrue(
                            standardComponentVariables[tp][vname]['doc'] in vr)
                        default = \
                            standardComponentVariables[tp][vname]['default']
                        if default is None:
                            default = 'None'
                        self.assertTrue(
                            vr.endswith(" [default: '%s']" % default))
        finally:
            pass

    def test_stdcomp_missing_parameters(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        types = [
            # "absorber",           # +-
            # "beamstop",           # +
            # "beamtimeid",         #
            # "chcut",              #
            "collect2",             # +
            "collect3",             # +
            "common2",              #
            "common3",              #
            # "datasignal",         #
            # "dcm",                #
            # "default",            # +
            # "defaultinstrument",  # .
            # "defaultsample",      # .
            # "empty",              #
            # "keithley",           #
            # "maia",               #
            # "maiadimension",      #
            # "maiaflux",           #
            # "pinhole",            #
            # "pointdet",           #
            # "qbpm",               #
            # "samplehkl",          #
            # "slit",               #
            # "source",             # +
            # "undulator"           #
        ]

        args = [
            ('nxscreate stdcomp %s -c cptest -t ' % self.flags).split(),
            ('nxscreate stdcomp %s --component cptest --type '
             % self.flags).split(),
        ]

        totest = []
        try:
            for tp in types:
                for arg in args:
                    cp = "cptest"
                    skip = False
                    if self.cpexists(cp):
                        skip = True
                    if not skip:
                        totest.append(cp)

                        cmd = list(arg)
                        cmd.append(tp)
                        # print(tp)
                        vl, er, txt = self.runtestexcept(cmd, SystemExit)
                        if er:
                            self.assertEqual(
                                "Info: NeXus hasn't been setup yet. \n\n", er)
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)
                        # print(txt)
                        lines = vl.split("\n")
                        # self.assertEqual(lines[0], "OUTPUT DIR: .")
                        self.assertEqual(lines[-1], "")
                        self.assertTrue("MISSING" in vl)
                        self.assertTrue("WARNING: " in vl)
                        self.assertTrue(" cannot be created without " in vl)
                        if self.cpexists(cp):
                            self.deletecp(cp)
        finally:
            for cp in totest:
                if self.cpexists(cp):
                    self.deletecp(cp)

    def test_stdcomp_missing_mand_parameters(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        types = [
            "absorber",             # +-
            # "beamstop",           # +
            # "beamtimeid",         #
            # "chcut",              #
            # "collect2",           # +
            # "collect3",           # +
            # "common2",            #
            # "common3",            #
            # "datasignal",         #
            # "dcm",                #
            # "default",            # +
            # "defaultinstrument",  # .
            # "defaultsample",      # .
            # "empty",              #
            # "keithley",           #
            # "maia",               #
            # "maiadimension",      #
            # "maiaflux",           #
            # "pinhole",            #
            # "pointdet",           #
            "qbpm",                 #
            # "samplehkl",          #
            # "slit",               #
            # "source",             # +
            # "undulator"           #
        ]

        args = [
            ('nxscreate stdcomp %s -c cptest -t ' % self.flags).split(),
            ('nxscreate stdcomp %s --component cptest --type '
             % self.flags).split(),
        ]

        totest = []
        try:
            for tp in types:
                for arg in args:
                    cp = "cptest"
                    skip = False
                    if self.cpexists(cp):
                        skip = True
                    if not skip:
                        totest.append(cp)

                        cmd = list(arg)
                        cmd.append(tp)
                        # print(tp)
                        vl, er = self.runtest(cmd)
                        if er:
                            self.assertEqual(
                                "Info: NeXus hasn't been setup yet. \n\n", er)
                        else:
                            self.assertEqual('', er)
                        self.assertTrue(vl)
                        # print(txt)
                        lines = vl.split("\n")
                        # self.assertEqual(lines[0], "OUTPUT DIR: .")
                        self.assertEqual(lines[-1], "")
                        self.assertTrue("MISSING" in vl)
                        self.assertTrue("WARNING: " in vl)
                        self.assertTrue(" cannot be created without " in vl)
                        if self.cpexists(cp):
                            self.deletecp(cp)
        finally:
            for cp in totest:
                if self.cpexists(cp):
                    self.deletecp(cp)

    def test_stdcomp_absorber(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t absorber -c absorber1 '
                 ' position mot01 '
                 ' %s' % self.flags).split(),
                [
                    ['absorber1'],
                    ['absorber1_foil', 'absorber1_thickness']
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry">\n'
                     '    <group name="instrument" type="NXinstrument">\n'
                     '      <group name="absorber1" type="NXattenuator">\n'
                     '        <group name="collection" type="NXcollection">\n'
                     '          <field name="slidersin_position" '
                     'type="NX_FLOAT64" units="">\n'
                     '          <strategy mode="INIT" />'
                     '$datasources.mot01</field>\n'
                     '\t</group>\n'
                     '        </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="absorber1_foil" type="PYEVAL">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import absorber\n'
                     'ds.result = absorber.foil('
                     'ds.mot01, \'["Ag", "Ag", "Ag", "Ag", "", "Al", "Al", '
                     '"Al", "Al"]\')'
                     '\n    </result>\n'
                     ' $datasources.mot01</datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="absorber1_thickness"'
                     ' type="PYEVAL">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import absorber\n'
                     'ds.result = absorber.thickness('
                     'ds.mot01, '
                     '\'[0.5, 0.05, 0.025, 0.0125, 0, 0.1, 0.3, 0.5, 1.0]\')\n'
                     '    </result>\n'
                     ' $datasources.mot01</datasource>\n'
                     '</definition>'],
                ],
            ],
            [
                ('nxscreate stdcomp --type absorber --component absorber1 '
                 ' position mot01 '
                 ' y y '
                 ' attenfactor afactor '
                 ' foil myfoil '
                 ' thickness tkns '
                 ' foillist ["Ag","","Al"] '
                 ' thicknesslist  [0.5,0,1.0] '
                 ' distance 0.5 '
                 ' distanceoffset [0,1,2] '
                 ' dependstop distance '
                 ' transformations transformations '
                 ' %s' % self.flags).split(),
                [
                    ['absorber1'],
                    ['absorber1_foil', 'absorber1_thickness']
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <group type="NXentry" '
                     'name="$var.entryname#\'scan\'$var.serialno">\n'
                     '    <group type="NXinstrument" name="instrument">\n'
                     '      <group type="NXattenuator" name="absorber1">\n'
                     '        <field type="NX_CHAR" name="type">\n'
                     '          <strategy mode="INIT"/>$datasources.myfoil'
                     '<dimensions rank="1"/>\n'
                     '\t</field>\n'
                     '        <field type="NX_CHAR" name="thickness">\n'
                     '          <strategy mode="INIT"/>$datasources.tkns'
                     '<dimensions rank="1"/>\n'
                     '\t</field>\n'
                     '        <field units="" type="NX_FLOAT" '
                     'name="attenuator_transmission">\n'
                     '          <strategy mode="INIT"/>$datasources.afactor'
                     '</field>\n'
                     '\t<group type="NXcollection" name="collection">\n'
                     '          <field units="" type="NX_FLOAT64" '
                     'name="slidersin_position">\n'
                     '          <strategy mode="INIT"/>$datasources.mot01'
                     '</field>\n'
                     '\t</group>\n'
                     '        <group type="NXtransformations" '
                     'name="transformations">\n'
                     '          <field depends_on="distance" units="mm" '
                     'type="NX_FLOAT64" name="y">\n'
                     '            <strategy mode="INIT"/>$datasources.y\n'
                     '\t    '
                     '<attribute type="NX_CHAR" name="transformation_type">'
                     'translation<strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '            <attribute type="NX_FLOAT64" name="vector">'
                     '0 1 0\n'
                     '\t    <strategy mode="INIT"/>\n'
                     '            <dimensions rank="1">\n'
                     '\t      <dim value="3" index="1"/>\n'
                     '            </dimensions>\n'
                     '            </attribute>\n'
                     '          </field>\n'
                     '          <field offset_units="m" units="m" '
                     'type="NX_FLOAT64" name="distance" '
                     'transformation_type="translation">0.5'
                     '<strategy mode="INIT"/>\n'
                     '            <attribute type="NX_FLOAT64" name="vector">'
                     '0 0 1<dimensions rank="1">\n'
                     '                <dim value="3" index="1"/>\n'
                     '              </dimensions>\n'
                     '              <strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '            <attribute type="NX_FLOAT64" name="offset">'
                     '[0,1,2]<dimensions rank="1">\n'
                     '                <dim value="3" index="1"/>\n'
                     '              </dimensions>\n'
                     '              <strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '          </field>\n'
                     '        </group>\n'
                     '        <field type="NX_CHAR" name="depends_on">'
                     'transformations/distance<strategy mode="INIT"/>\n'
                     '        </field>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>\n'],
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" name="absorber1_foil">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import absorber\n'
                     'ds.result = absorber.foil('
                     'ds.mot01, \'["Ag","","Al"]\')\n'
                     '    </result>\n'
                     ' $datasources.mot01</datasource>\n'
                     '</definition>\n',
                     '<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL"'
                     ' name="absorber1_thickness">\n'
                     '    <result name="result">\n'
                     'from nxstools.pyeval import absorber\n'
                     'ds.result = absorber.thickness('
                     'ds.mot01, \'[0.5,0,1.0]\')\n'
                     '    </result>\n'
                     ' $datasources.mot01</datasource>\n'
                     '</definition>\n'],
                ],
            ],
        ]

        self.checkxmls(args)

    def test_stdcomp_beamstop(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t beamstop -c testbeamstop1 %s' %
                 self.flags).split(),
                [
                    ['testbeamstop1'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno" '
                     'type="NXentry">\n'
                     '    <group name="instrument" type="NXinstrument">\n'
                     '      <group name="testbeamstop1" type="NXbeam_stop">\n'
                     '\t<field name="description" type="NX_CHAR">\n'
                     '            <strategy mode="INIT" />circular</field>\n'
                     '        <field name="depends_on" type="NX_CHAR">'
                     'transformations/y<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '        <group name="transformations" '
                     'type="NXtransformations">\n'
                     '          </group>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    [],
                ],
            ],
            [
                ('nxscreate stdcomp --type beamstop '
                 ' --entryname myentry '
                 ' --insname myinstrument '
                 '--component testbeamstop2 %s' %
                 self.flags).split(),
                [
                    ['testbeamstop2'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'myentry\'$var.serialno" '
                     'type="NXentry">\n'
                     '    <group name="myinstrument" type="NXinstrument">\n'
                     '      <group name="testbeamstop2" type="NXbeam_stop">\n'
                     '\t<field name="description" type="NX_CHAR">\n'
                     '            <strategy mode="INIT" />circular</field>\n'
                     '        <field name="depends_on" type="NX_CHAR">'
                     'transformations/y<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '        <group name="transformations" '
                     'type="NXtransformations">\n'
                     '          </group>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    [],
                ],
            ],
            [
                ('nxscreate stdcomp -t beamstop -c testbeamstop3 '
                 ' -y myentry '
                 ' -i myinstrument '
                 ' description linear '
                 ' x mot01 '
                 ' xsign -'
                 ' y mot02 '
                 ' z mot03 '
                 ' %s' % self.flags).split(),
                [
                    ['testbeamstop3'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <group type="NXentry" '
                     'name="$var.entryname#\'myentry\'$var.serialno">\n'
                     '    <group type="NXinstrument" name="myinstrument">\n'
                     '      <group type="NXbeam_stop" name="testbeamstop3">\n'
                     '\t<field type="NX_CHAR" name="description">\n'
                     '            <strategy mode="INIT"/>linear</field>\n'
                     '        <field type="NX_CHAR" name="depends_on">'
                     'transformations/y<strategy mode="INIT"/>\n'
                     '        </field>\n'
                     '        <group type="NXtransformations" '
                     'name="transformations">\n'
                     '          <field depends_on="x" units="mm" '
                     'type="NX_FLOAT64" name="y">\n'
                     '            <strategy mode="INIT"/>$datasources.mot02\n'
                     '\t    '
                     '<attribute type="NX_CHAR" name="transformation_type">'
                     'translation<strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '            <attribute type="NX_FLOAT64" name="vector">'
                     '0 1 0\n'
                     '\t    <strategy mode="INIT"/>\n'
                     '            <dimensions rank="1">\n'
                     '\t      <dim value="3" index="1"/>\n'
                     '            </dimensions>\n'
                     '            </attribute>\n'
                     '          </field>\n'
                     '          <field depends_on="z" units="mm" '
                     'type="NX_FLOAT64" name="x">\n'
                     '            <strategy mode="INIT"/>$datasources.mot01\n'
                     '\t    '
                     '<attribute type="NX_CHAR" name="transformation_type">'
                     'translation<strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '            <attribute type="NX_FLOAT64" name="vector">'
                     '-1 0 0\n\t    <strategy mode="INIT"/>\n'
                     '            <dimensions rank="1">\n'
                     '\t      <dim value="3" index="1"/>\n'
                     '            </dimensions>\n'
                     '            </attribute>\n'
                     '          </field>\n'
                     '          '
                     '<field units="mm" type="NX_FLOAT64" name="z">\n'
                     '            <strategy mode="INIT"/>$datasources.mot03\n'
                     '\t    '
                     '<attribute type="NX_CHAR" name="transformation_type">'
                     'translation<strategy mode="INIT"/>\n'
                     '            </attribute>\n'
                     '            <attribute type="NX_FLOAT64" name="vector">'
                     '0 0 1\n\t    <strategy mode="INIT"/>\n'
                     '            <dimensions rank="1">\n'
                     '\t      <dim value="3" index="1"/>\n'
                     '            </dimensions>\n'
                     '            </attribute>\n'
                     '          </field>\n'
                     '        </group>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>\n'],
                    [],
                ],
            ],
        ]

        self.checkxmls(args)

    def test_stdcomp_default_mandatory(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t default -c default -m '
                 ' %s' % self.flags).split(),
                [
                    ['default', 'defaultinstrument', 'defaultsample'],
                    ['title', 'start_time', 'sample_name',
                     'nexdatas_version', 'nexdatas_configuration',
                     'end_time', 'chemical_formula', 'beamtime_id',
                     'beamtime_filename']
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry" />\n'
                     '  $components.defaultinstrument\n'
                     '  $components.defaultsample\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry">\n'
                     '    <field name="experiment_identifier" '
                     'type="NX_CHAR">\n'
                     '      <doc>Beamtime ID. From this ID everything else '
                     'can be derived from the DESY databases including the '
                     'Proposal as well as the scientists performing '
                     'the experiment, the local contact, and so on.\n'
                     'The beamtime ID at DESY is an 8 digit number.R</doc>\n'
                     '      <strategy mode="INIT" />$datasources.beamtime_id\n'
                     '      <attribute type="NX_CHAR" '
                     'name="beamtime_filename">'
                     '        <strategy mode="INIT" />'
                     '$datasources.beamtime_filename</attribute>\n'
                     '</field>\n'
                     '    <field name="start_time" type="NX_DATE_TIME">\n'
                     '      <doc>time stamp when the experiment has started.'
                     '</doc>\n'
                     '      <strategy mode="INIT" />$datasources.start_time'
                     '</field>\n'
                     '    <field name="end_time" type="NX_DATE_TIME">\n'
                     '      <doc>'
                     'end time - timestamp when the experiment stopped.'
                     '</doc>\n'
                     '      <strategy mode="FINAL" />$datasources.end_time'
                     '</field>\n'
                     '    <group name="instrument" type="NXinstrument">\n'
                     '      <group name="source" type="NXsource">\n'
                     '        <doc>'
                     'generic description of the PETRA III storage ring'
                     '</doc>\n'
                     '        <field name="name" short_name="PETRAIII"'
                     ' type="NX_CHAR">\n'
                     '          <strategy mode="INIT" />PETRA III</field>\n'
                     '        <field name="type" type="NX_CHAR">'
                     'Synchrotron X-ray Source<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '        <field name="probe" type="NX_CHAR">'
                     'x-ray<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '      </group>\n'
                     '      <field name="name" '
                     'short_name="P09" type="NX_CHAR">'
                     'P09 Resonant Scattering and Diffraction beamline'
                     '<strategy mode="INIT" />\n'
                     '      </field>\n'
                     '    </group>\n'
                     '    <field name="program_name" '
                     'scan_command="$var.scan_title" scan_id="$var.scan_id" '
                     'npoints="$var.npoints" count_time="$var.count_time" '
                     'beamtime_id="$var.beamtime_id" '
                     'measurement_group="$var.measurement_group" '
                     'measurement_group_channels="$var.mgchannels" '
                     'nexus_components="$var.nexus_components" '
                     'nexus_step_datasources="$var.nexus_step_datasources" '
                     'nexus_init_datasources="$var.nexus_init_datasources" '
                     'type="NX_CHAR">NexDaTaS<attribute name="version" '
                     'type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.nexdatas_version</attribute>\n'
                     '      <attribute name="configuration" type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.nexdatas_configuration</attribute>\n'
                     '      <strategy mode="INIT"/>\n'
                     '    </field>\n'
                     '    <field name="title" type="NX_CHAR">\n'
                     '      <strategy mode="INIT" />$datasources.title'
                     '</field>\n'
                     '  </group>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry">\n'
                     '    <group name="sample" type="NXsample">\n'
                     '      <field name="name" type="NX_CHAR">\n'
                     '\t<strategy mode="INIT" />$datasources.sample_name\n'
                     '      </field>\n'
                     '      <field name="chemical_formula" type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.chemical_formula\n'
                     '      </field>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="title" type="CLIENT">\n'
                     '    <record name="title" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="start_time" type="CLIENT">\n'
                     '    <record name="start_time" />\n'
                     '    <doc>'
                     'The start time is provided by the control client.'
                     '</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="sample_name" type="CLIENT">\n'
                     '    <record name="sample_name" />\n'
                     '    <doc>'
                     'Data source providing the name of the sample.</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="nexdatas_version" type="PYEVAL">\n'
                     '    <result name="result">\n'
                     'from nxswriter import __version__\n'
                     'ds.result = __version__   # from nxswriter\n'
                     '    </result>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource '
                     'name="nexdatas_configuration" type="TANGO">\n'
                     '    <record name="Version" />\n'
                     '    <device hostname="%s" member="attribute" '
                     'name="%s" port="%s" />\n'
                     '  </datasource>\n'
                     '</definition>' %
                     (self.host, self.device, self.port),
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="end_time" type="CLIENT">\n'
                     '    <record name="end_time" />\n'
                     '    <doc>'
                     'The end time is provided by the client after '
                     'the experiment is finished.</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="chemical_formula" type="CLIENT">\n'
                     '    <record name="chemical_formula" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="beamtime_id" type="CLIENT">\n'
                     '    <record name="beamtime_id" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" name="beamtime_filename">\n'
                     '    <result name="result">\n'
                     'ds.result = ""\n</result>\n'
                     '  </datasource>\n'
                     '</definition>']
                ],
            ],
            [
                ('nxscreate stdcomp --type default --component default'
                 ' --mandatory'
                 ' %s' % self.flags).split(),
                [
                    ['default', 'defaultinstrument', 'defaultsample'],
                    ['title', 'start_time', 'sample_name',
                     'nexdatas_version', 'nexdatas_configuration',
                     'end_time', 'chemical_formula', 'beamtime_id',
                     'beamtime_filename']
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry" />\n'
                     '  $components.defaultinstrument\n'
                     '  $components.defaultsample\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry">\n'
                     '    <field name="experiment_identifier" '
                     'type="NX_CHAR">\n'
                     '      <doc>Beamtime ID. From this ID everything else '
                     'can be derived from the DESY databases including the '
                     'Proposal as well as the scientists performing '
                     'the experiment, the local contact, and so on.\n'
                     'The beamtime ID at DESY is an 8 digit number.R</doc>\n'
                     '      <strategy mode="INIT" />$datasources.beamtime_id\n'
                     '      <attribute type="NX_CHAR" '
                     'name="beamtime_filename">'
                     '        <strategy mode="INIT" />'
                     '$datasources.beamtime_filename</attribute>\n'
                     '</field>\n'
                     '    <field name="start_time" type="NX_DATE_TIME">\n'
                     '      <doc>time stamp when the experiment has started.'
                     '</doc>\n'
                     '      <strategy mode="INIT" />$datasources.start_time'
                     '</field>\n'
                     '    <field name="end_time" type="NX_DATE_TIME">\n'
                     '      <doc>'
                     'end time - timestamp when the experiment stopped.'
                     '</doc>\n'
                     '      <strategy mode="FINAL" />$datasources.end_time'
                     '</field>\n'
                     '    <group name="instrument" type="NXinstrument">\n'
                     '      <group name="source" type="NXsource">\n'
                     '        <doc>'
                     'generic description of the PETRA III storage ring'
                     '</doc>\n'
                     '        <field name="name" short_name="PETRAIII"'
                     ' type="NX_CHAR">\n'
                     '          <strategy mode="INIT" />PETRA III</field>\n'
                     '        <field name="type" type="NX_CHAR">'
                     'Synchrotron X-ray Source<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '        <field name="probe" type="NX_CHAR">'
                     'x-ray<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '      </group>\n'
                     '      <field name="name" '
                     'short_name="P09" type="NX_CHAR">'
                     'P09 Resonant Scattering and Diffraction beamline'
                     '<strategy mode="INIT" />\n'
                     '      </field>\n'
                     '    </group>\n'
                     '    <field name="program_name" '
                     'scan_command="$var.scan_title" scan_id="$var.scan_id" '
                     'npoints="$var.npoints" count_time="$var.count_time" '
                     'beamtime_id="$var.beamtime_id" '
                     'measurement_group="$var.measurement_group" '
                     'measurement_group_channels="$var.mgchannels" '
                     'nexus_components="$var.nexus_components" '
                     'nexus_step_datasources="$var.nexus_step_datasources" '
                     'nexus_init_datasources="$var.nexus_init_datasources" '
                     'type="NX_CHAR">NexDaTaS<attribute name="version" '
                     'type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.nexdatas_version</attribute>\n'
                     '      <attribute name="configuration" type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.nexdatas_configuration</attribute>\n'
                     '      <strategy mode="INIT"/>\n'
                     '    </field>\n'
                     '    <field name="title" type="NX_CHAR">\n'
                     '      <strategy mode="INIT" />$datasources.title'
                     '</field>\n'
                     '  </group>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno"'
                     ' type="NXentry">\n'
                     '    <group name="sample" type="NXsample">\n'
                     '      <field name="name" type="NX_CHAR">\n'
                     '\t<strategy mode="INIT" />$datasources.sample_name\n'
                     '      </field>\n'
                     '      <field name="chemical_formula" type="NX_CHAR">\n'
                     '        <strategy mode="INIT" />'
                     '$datasources.chemical_formula\n'
                     '      </field>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="title" type="CLIENT">\n'
                     '    <record name="title" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="start_time" type="CLIENT">\n'
                     '    <record name="start_time" />\n'
                     '    <doc>'
                     'The start time is provided by the control client.'
                     '</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="sample_name" type="CLIENT">\n'
                     '    <record name="sample_name" />\n'
                     '    <doc>'
                     'Data source providing the name of the sample.</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="nexdatas_version" type="PYEVAL">\n'
                     '    <result name="result">\n'
                     'from nxswriter import __version__\n'
                     'ds.result = __version__   # from nxswriter\n'
                     '    </result>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource '
                     'name="nexdatas_configuration" type="TANGO">\n'
                     '    <record name="Version" />\n'
                     '    <device hostname="%s" member="attribute" '
                     'name="%s" port="%s" />\n'
                     '  </datasource>\n'
                     '</definition>' %
                     (self.host, self.device, self.port),
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="end_time" type="CLIENT">\n'
                     '    <record name="end_time" />\n'
                     '    <doc>'
                     'The end time is provided by the client after '
                     'the experiment is finished.</doc>\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="chemical_formula" type="CLIENT">\n'
                     '    <record name="chemical_formula" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource name="beamtime_id" type="CLIENT">\n'
                     '    <record name="beamtime_id" />\n'
                     '  </datasource>\n'
                     '</definition>',
                     '<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" name="beamtime_filename">\n'
                     '    <result name="result">\n'
                     'ds.result = ""\n</result>\n'
                     '  </datasource>\n'
                     '</definition>']
                ],
            ],
        ]

        self.checkxmls(args, True)

    def test_stdcomp_source_nolower(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t source -c Source '
                 ' %s' % self.flags).split(),
                [
                    ['source'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\' encoding=\'utf8\'?>\n'
                     '<definition>\n'
                     '  <group name="$var.entryname#\'scan\'$var.serialno" '
                     'type="NXentry">\n'
                     '    <group name="instrument" type="NXinstrument">\n'
                     '      <group name="source" type="NXsource">\n'
                     '        <doc>generic description of the storage ring'
                     '</doc>\n'
                     '        <field name="mode" type="NX_CHAR">'
                     'Multi Bunch<strategy mode="INIT" />\n'
                     '        </field>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>'],
                    [],
                ],
            ],
            [
                ('nxscreate stdcomp --type source --component Source '
                 ' --nolower '
                 ' beamcurrent bcurrent '
                 ' bunchmode Single_Bunch '
                 ' numberofbunches nob '
                 ' sourceenergy senergy '
                 ' %s' % self.flags).split(),
                [
                    ['Source'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <group type="NXentry" '
                     'name="$var.entryname#\'scan\'$var.serialno">\n'
                     '    <group type="NXinstrument" name="instrument">\n'
                     '      <group type="NXsource" name="source">\n'
                     '        <doc>generic description of the storage ring'
                     '</doc>\n'
                     '        <field units="mA" type="NX_FLOAT" '
                     'name="current">\n'
                     '          <doc>storage ring current</doc>\n'
                     '          <strategy mode="INIT" '
                     'canfail="true"/>$datasources.bcurrent\n'
                     '\t</field>\n'
                     '        <field units="GeV" type="NX_FLOAT" '
                     'name="energy">\n'
                     '\t  <doc>beam energy</doc>\n'
                     '          <strategy mode="INIT" canfail="true"/>'
                     '$datasources.senergy\n'
                     '\t</field>\n'
                     '\t<field type="NX_INT64" name="number_of_bunches">\n'
                     '          <strategy mode="INIT" canfail="true"/>'
                     '$datasources.nob\n'
                     '\t</field>\n'
                     '        <field type="NX_CHAR" name="mode">Single_Bunch'
                     '<strategy mode="INIT"/>\n'
                     '        </field>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>\n'
                     ''],
                    [],
                ],
            ],
            [
                ('nxscreate stdcomp -t source -c Source -n '
                 ' beamcurrent bcurrent '
                 ' bunchmode Single_Bunch '
                 ' numberofbunches nob '
                 ' sourceenergy senergy '
                 ' %s' % self.flags).split(),
                [
                    ['Source'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <group type="NXentry" '
                     'name="$var.entryname#\'scan\'$var.serialno">\n'
                     '    <group type="NXinstrument" name="instrument">\n'
                     '      <group type="NXsource" name="source">\n'
                     '        <doc>generic description of the storage ring'
                     '</doc>\n'
                     '        <field units="mA" type="NX_FLOAT" '
                     'name="current">\n'
                     '          <doc>storage ring current</doc>\n'
                     '          <strategy mode="INIT" '
                     'canfail="true"/>$datasources.bcurrent\n'
                     '\t</field>\n'
                     '        <field units="GeV" type="NX_FLOAT" '
                     'name="energy">\n'
                     '\t  <doc>beam energy</doc>\n'
                     '          <strategy mode="INIT" canfail="true"/>'
                     '$datasources.senergy\n'
                     '\t</field>\n'
                     '\t<field type="NX_INT64" name="number_of_bunches">\n'
                     '          <strategy mode="INIT" canfail="true"/>'
                     '$datasources.nob\n'
                     '\t</field>\n'
                     '        <field type="NX_CHAR" name="mode">Single_Bunch'
                     '<strategy mode="INIT"/>\n'
                     '        </field>\n'
                     '      </group>\n'
                     '    </group>\n'
                     '  </group>\n'
                     '</definition>\n'
                     ''],
                    [],
                ],
            ],
        ]

        self.checkxmls(args)

    def test_stdcomp_collect2_overwrite_false(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t collect2 -c myslits '
                 ' first slit1 '
                 ' second slit2 '
                 ' %s' % self.flags).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '</definition>\n'],
                    [''],
                ],
                ('nxscreate stdcomp -t collect2 -c myslits '
                 ' first mslit1 '
                 ' second mslit2 '
                 ' %s' % self.flags).split(),
                ('nxscreate stdcomp --type collect2 --component myslits '
                 ' first slit1 '
                 ' second slit2 '
                 ' %s' % self.flags).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '</definition>\n'],
                    [''],
                ],
                ('nxscreate stdcomp -t collect2 -c myslits '
                 ' first mslit1 '
                 ' second mslit2 '
                 ' %s' % self.flags).split(),
            ],
        ]

        dstotest = []
        cptotest = []
        try:
            for arg in args:
                skip = False
                for cp in arg[1][0]:
                    if self.cpexists(cp):
                        skip = True
                for ds in arg[1][1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1][1]:
                        dstotest.append(ds)
                    for cp in arg[1][0]:
                        cptotest.append(cp)

                    vl, er = self.runtest(arg[0])
                    # print(vl)
                    if er:
                        self.assertEqual(
                            "Info: NeXus hasn't been setup yet. \n\n", er)
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    for i, ds in enumerate(arg[1][1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][1][i], xml)
                    for i, cp in enumerate(arg[1][0]):
                        xml = self.getcp(cp)
                        self.assertEqual(arg[2][0][i], xml)

                    vl, er, txt = self.runtestexcept(arg[3], SystemExit)

                    for i, ds in enumerate(arg[1][1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][1][i], xml)
                    for i, cp in enumerate(arg[1][0]):
                        xml = self.getcp(cp)
                        self.assertEqual(arg[2][0][i], xml)

                    for ds in arg[1][1]:
                        self.deleteds(ds)
                    for cp in arg[1][0]:
                        self.deletecp(cp)

        finally:
            pass
            for cp in cptotest:
                if self.cpexists(cp):
                    self.deletecp(cp)
            for ds in dstotest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_stdcomp_collect3_overwrite_true(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        args = [
            [
                ('nxscreate stdcomp -t collect3 -c myslits '
                 ' first mslit1 '
                 ' second mslit2 '
                 ' third mslit3 '
                 ' %s' % self.flags).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '  $components.slit3\n'
                     '</definition>\n'],
                    [''],
                ],
                ('nxscreate stdcomp -t collect3 -c myslits -o '
                 ' first slit1 '
                 ' second slit2 '
                 ' third slit3 '
                 ' %s' % self.flags).split(),
            ],
            [
                ('nxscreate stdcomp --type collect3 --component myslits '
                 ' first mslit1 '
                 ' second mslit2 '
                 ' third mslit3 '
                 ' %s' % self.flags).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '  $components.slit3\n'
                     '</definition>\n'],
                    [''],
                ],
                ('nxscreate stdcomp --type collect3 --component myslits '
                 ' --overwrite '
                 ' first slit1 '
                 ' second slit2 '
                 ' third slit3 '
                 ' %s' % self.flags).split(),
            ],
        ]

        dstotest = []
        cptotest = []
        try:
            for arg in args:
                skip = False
                for cp in arg[1][0]:
                    if self.cpexists(cp):
                        skip = True
                for ds in arg[1][1]:
                    if self.dsexists(ds):
                        skip = True
                if not skip:
                    for ds in arg[1][1]:
                        dstotest.append(ds)
                    for cp in arg[1][0]:
                        cptotest.append(cp)

                    vl, er = self.runtest(arg[0])
                    # print(vl)
                    if er:
                        self.assertEqual(
                            "Info: NeXus hasn't been setup yet. \n\n", er)
                    else:
                        self.assertEqual('', er)
                    self.assertTrue(vl)

                    vl, er = self.runtest(arg[3])

                    for i, ds in enumerate(arg[1][1]):
                        xml = self.getds(ds)
                        self.assertEqual(arg[2][1][i], xml)
                    for i, cp in enumerate(arg[1][0]):
                        xml = self.getcp(cp)
                        self.assertEqual(arg[2][0][i], xml)

                    for ds in arg[1][1]:
                        self.deleteds(ds)
                    for cp in arg[1][0]:
                        self.deletecp(cp)

        finally:
            pass
            for cp in cptotest:
                if self.cpexists(cp):
                    self.deletecp(cp)
            for ds in dstotest:
                if self.dsexists(ds):
                    self.deleteds(ds)

    def test_stdcomp_missing_parameters_package(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        types = [
            "collect4",
            "common4",
        ]

        if __name__ == 'test.NXSCreateStdCompFSTest':
            args = [
                ('nxscreate stdcomp %s -p test.nxsextrasp00 -c cptest -t '
                 % self.flags).split(),
                ('nxscreate stdcomp %s --xml-package test.nxsextrasp00 '
                 ' --component cptest --type '
                 % self.flags).split(),
            ]
        else:
            args = [
                ('nxscreate stdcomp %s -p nxsextrasp00 -c cptest -t '
                 % self.flags).split(),
                ('nxscreate stdcomp %s --xml-package nxsextrasp00 '
                 ' --component cptest --type '
                 % self.flags).split(),
            ]

        totest = []
        try:
            for tp in types:
                for arg in args:
                    cp = "cptest"
                    skip = False
                    if self.cpexists(cp):
                        skip = True
                    if not skip:
                        totest.append(cp)

                        cmd = list(arg)
                        cmd.append(tp)
                        # print(tp)
                        vl, er, txt = self.runtestexcept(cmd, SystemExit)

                        lines = vl.split("\n")
                        # self.assertEqual(lines[0], "OUTPUT DIR: .")
                        self.assertEqual(lines[-1], "")
                        self.assertTrue("MISSING" in vl)
                        self.assertTrue("WARNING: " in vl)
                        self.assertTrue(" cannot be created without " in vl)
                        if self.cpexists(cp):
                            self.deletecp(cp)
        finally:
            for cp in totest:
                if self.cpexists(cp):
                    self.deletecp(cp)

    def test_stdcomp_collect4(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        if __name__ == 'test.NXSCreateStdCompFSTest':
            pname = 'test.nxsextrasp00'
        else:
            pname = 'nxsextrasp00'
        args = [
            [
                ('nxscreate stdcomp -t collect4 -c myslits '
                 '-p %s '
                 ' first slit1 '
                 ' second slit2 '
                 ' third slit3 '
                 ' fourth slit4 '
                 ' %s' % (pname, self.flags)).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '  $components.slit3\n'
                     '  $components.slit4\n'
                     '</definition>\n'],
                    [''],
                ],
            ],
            [
                ('nxscreate stdcomp --type collect4 --component myslits '
                 ' --xml-package %s '
                 ' first slit1 '
                 ' second slit2 '
                 ' third slit3 '
                 ' fourth slit4 '
                 ' %s' % (pname, self.flags)).split(),
                [
                    ['myslits'],
                    []
                ],
                [
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  $components.slit1\n'
                     '  $components.slit2\n'
                     '  $components.slit3\n'
                     '  $components.slit4\n'
                     '</definition>\n'],
                    [''],
                ],
            ],
        ]

        self.checkxmls(args)

    def test_stdcomp_common4(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        if __name__ == 'test.NXSCreateStdCompFSTest':
            pname = 'test.nxsextrasp00'
        else:
            pname = 'nxsextrasp00'
        args = [
            [
                ('nxscreate stdcomp -t common4 -c myslit '
                 '-p %s '
                 ' dds slit1 '
                 ' ods1 slit2 '
                 ' ods2 slit3 '
                 ' ods3 slit4 '
                 ' %s' % (pname, self.flags)).split(),
                [
                    [],
                    ['myslit_common']
                ],
                [
                    [],
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" name="myslit_common">\n'
                     '    <result name="result">\n'
                     'ds.result = ds.slit1</result>\n'
                     ' $datasources.slit1\n'
                     ' $datasources.slit2\n'
                     ' $datasources.slit3\n'
                     ' $datasources.slit4</datasource>\n'
                     '</definition>\n'],
                ],
            ],
            [
                ('nxscreate stdcomp --type common4 --component myslit '
                 ' --xml-package %s '
                 ' dds slit1 '
                 ' ods1 slit2 '
                 ' ods2 slit3 '
                 ' ods3 slit4 '
                 ' %s' % (pname, self.flags)).split(),
                [
                    [],
                    ['myslit_common']
                ],
                [
                    [],
                    ['<?xml version=\'1.0\'?>\n'
                     '<definition>\n'
                     '  <datasource type="PYEVAL" name="myslit_common">\n'
                     '    <result name="result">\n'
                     'ds.result = ds.slit1</result>\n'
                     ' $datasources.slit1\n'
                     ' $datasources.slit2\n'
                     ' $datasources.slit3\n'
                     ' $datasources.slit4</datasource>\n'
                     '</definition>\n'],
                ],
            ],
        ]

        self.checkxmls(args)

    def test_stdcomp_typelist_package(self):
        """ test nxsccreate stdcomp file system
        """
        fun = sys._getframe().f_code.co_name
        print("Run: %s.%s() " % (self.__class__.__name__, fun))

        if __name__ == 'test.NXSCreateStdCompFSTest':
            pname = 'test.nxsextrasp00'
        else:
            pname = 'nxsextrasp00'
        args = [
            [
                ('nxscreate stdcomp '
                 ' -p %s '
                 ' %s' % (pname, self.flags)).split(),
                ('nxscreate stdcomp '
                 ' --xml-package %s '
                 ' %s' % (pname, self.flags)).split(),
            ],
        ]

        try:
            for arg in args:
                vl, er = self.runtest(arg[0])

                if er:
                    self.assertEqual(
                        "Info: NeXus hasn't been setup yet. \n\n", er)
                else:
                    self.assertEqual('', er)
                self.assertTrue(vl)
                lines = vl.split("\n")
                self.assertEqual(lines[-3], "POSSIBLE COMPONENT TYPES: ")
                self.assertEqual(
                    lines[-2].split(),
                    ["collect4", "common4"])
                self.assertEqual(
                    lines[-2].split(),
                    sorted(nxsextrasp00.standardComponentVariables.keys()))
                self.assertEqual(lines[-1], "")
        finally:
            pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
# \package test nexdatas
# \file XMLConfiguratorTest.py
# unittests for field Tags running Tango Server
#
import unittest
import sys


try:
    import NXSCreateOnlineDSDB_test
except Exception:
    from . import NXSCreateOnlineDSDB_test


if sys.version_info > (3,):
    unicode = str
    long = int


# test fixture
class NXSCreateOnlineDSDB2Test(
        NXSCreateOnlineDSDB_test.NXSCreateOnlineDSDBTest):

    # constructor
    # \param methodName name of the test method
    def __init__(self, methodName):
        NXSCreateOnlineDSDB_test.NXSCreateOnlineDSDBTest.__init__(
            self, methodName)

        self.flags = " -b -r testp09/testmcs/testr228"


if __name__ == '__main__':
    unittest.main()
